@@ -74,6 +74,21 @@ def default_log_path():
     return Path.home() / "negmas" / "logs"
 
 
+DB_FOLDER = default_log_path().parent / "runsdb"
+DB_NAME = "rundb.csv"
+
+
+def save_run_info(
+    name: str, log_path: Path, type_: str = "world", path: Path = DB_FOLDER
+):
+    try:
+        path.mkdir(parents=True, exist_ok=True)
+        with open(path / DB_NAME, "a") as f:
+            f.write(f"{name},{type_},{str(log_path)}\n")
+    except Exception as e:
+        print(f"Failed to save run info: {str(e)}")
+
+
 def default_tournament_path():
     """The default path to store tournament run info"""
     return default_log_path() / "tournaments"
@@ -247,7 +262,7 @@ def main():
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -515,7 +530,7 @@ def tournament2019(
 )
 @click.option(
     "--competitors",
-    default="DecentralizingAgent;BuyCheapSellExpensiveAgent;RandomAgent",
+    default="SatisficerAgent;DecentralizingAgent;BuyCheapSellExpensiveAgent;RandomAgent;SatisficerAgent",
     help="A semicolon (;) separated list of agent types to use for the competition. You"
     " can also pass the special value default for the default builtin"
     " agents",
@@ -553,7 +568,7 @@ def tournament2019(
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -745,6 +760,7 @@ def tournament2020(
     end_time = humanize_time(perf_counter() - start)
     display_results(results, "median", output)
     print(f"Finished in {end_time}")
+    save_run_info(name, results.path, "tournament")
 
 
 def display_results(results, metric, file_name=None):
@@ -838,7 +854,7 @@ def _path(path) -> Path:
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -1116,13 +1132,15 @@ def run2019(
         print(exception)
         world.logdebug(exception)
         print(f"FAILED at step {world.current_step} of {world.n_steps}\n")
+    # else:
+    #     save_run_info(world.name, log_path=log_dir)
 
 
 @main.command(help="Run an SCML2020 world simulation")
 @click.option("--steps", default=10, type=int, help="Number of steps.")
 @click.option(
     "--competitors",
-    default="RandomAgent;BuyCheapSellExpensiveAgent;DecentralizingAgent;DoNothingAgent",
+    default="RandomAgent;BuyCheapSellExpensiveAgent;SatisficerAgent;DecentralizingAgent;DoNothingAgent",
     help="A semicolon (;) separated list of agent types to use for the competition.",
 )
 @click.option(
@@ -1141,7 +1159,7 @@ def run2019(
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -1409,18 +1427,25 @@ def run2020(
         print(exception)
         world.logdebug(exception)
         print(f"FAILED at step {world.current_step} of {world.n_steps}\n")
+    else:
+        save_run_info(world.name, log_dir, "world")
 
 
 DEFAULT_STD = (
-    "RandomAgent;BuyCheapSellExpensiveAgent;DecentralizingAgent;DoNothingAgent"
+    "RandomAgent;BuyCheapSellExpensiveAgent;SatisficerAgent;DecentralizingAgent;DoNothingAgent"
 )
-DEFAULT_STD_2021 = "MarketAwareDecentralizingAgent;scml.oneshot.agents.RandomOneShotAgent;DecentralizingAgent"
+DEFAULT_STD_2021 = "MarketAwareDecentralizingAgent;RandomAgent;SatisficerAgent;DecentralizingAgent"
+DEFAULT_2021_NONCOMPETITORS = [
+    # "scml.scml2020.agents.DecentralizingAgent",
+    "scml.scml2020.agents.BuyCheapSellExpensiveAgent",
+]
 # DEFAULT_STD_2021 = (
 #     "MarketAwareDecentralizingAgent;BuyCheapSellExpensiveAgent;DecentralizingAgent"
 # )
-DEFAULT_ONESHOT = "scml.scml2020.agents.DecentralizingAgent;RandomOneShotAgent;SyncRandomOneShotAgent;SingleAgreementRandomAgent;SingleAgreementAspirationAgent"
-DEFAULT_ONESHOT_FULL = [
+DEFAULT_ONESHOT = "GreedySyncAgent;SingleAgreementAspirationAgent"
+DEFAULT_ONESHOT_NONCOMPETITORS = [
     "scml.oneshot.agents.RandomOneShotAgent",
+    "scml.oneshot.agents.GreedyOneShotAgent",
     "scml.oneshot.agents.SingleAgreementAspirationAgent",
 ]
 
@@ -1448,7 +1473,7 @@ DEFAULT_ONESHOT_FULL = [
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -1478,6 +1503,11 @@ DEFAULT_ONESHOT_FULL = [
     help="A path to be added to PYTHONPATH in which all competitors are stored. You can path a : separated list of "
     "paths on linux/mac and a ; separated list in windows",
 )
+@click.option(
+    "--name",
+    default="",
+    help="World name. Used as the folder name in which it will be saved",
+)
 @click.option(
     "--world-config",
     type=click.Path(dir_okay=False, file_okay=True),
@@ -1485,6 +1515,12 @@ DEFAULT_ONESHOT_FULL = [
     multiple=False,
     help="A file to load extra configuration parameters for world simulations from.",
 )
+@click.option(
+    "--method",
+    type=click.Choice(["guaranteed_profit", "profitable"]),
+    default="profitable",
+    help="The method used for world generation",
+)
 @click_config_file.configuration_option()
 def run2021(
     steps,
@@ -1499,9 +1535,15 @@ def run2021(
     world_config,
     show_contracts,
     oneshot,
+    name,
+    method,
 ):
     if not competitors:
-        competitors = DEFAULT_ONESHOT if oneshot else DEFAULT_STD_2021
+        competitors = (
+            (DEFAULT_ONESHOT + ";" + ";".join(DEFAULT_ONESHOT_NONCOMPETITORS))
+            if oneshot
+            else (DEFAULT_STD_2021 + ";" + ";".join(DEFAULT_2021_NONCOMPETITORS))
+        )
     world_type = SCML2020OneShotWorld if oneshot else SCML2021World
     if time <= 0:
         time = None
@@ -1520,8 +1562,12 @@ def run2021(
 
     log_dir = _path(log)
 
-    world_name = unique_name(
-        base=f"scml2020{'oneshot' if oneshot else ''}", add_time=True, rand_digits=0
+    world_name = (
+        unique_name(
+            base=f"scml2020{'oneshot' if oneshot else ''}", add_time=True, rand_digits=0
+        )
+        if not name
+        else name
     )
     log_dir = log_dir / world_name
     log_dir = log_dir.absolute()
@@ -1554,6 +1600,7 @@ def run2021(
             name=world_name,
             ignore_agent_exceptions=not raise_exceptions,
             ignore_contract_execution_exceptions=not raise_exceptions,
+            method=method,
             **kwargs,
         )
     )
@@ -1727,6 +1774,8 @@ def run2021(
         print(exception)
         world.logdebug(exception)
         print(f"FAILED at step {world.current_step} of {world.n_steps}\n")
+    else:
+        save_run_info(world.name, log_dir, "world")
 
 
 @main.command(help="Runs an SCML2020 tournament")
@@ -1773,7 +1822,7 @@ def run2021(
 )
 @click.option(
     "--competitors",
-    default="DecentralizingAgent;BuyCheapSellExpensiveAgent;RandomAgent",
+    default="SatisficerAgent;DecentralizingAgent;BuyCheapSellExpensiveAgent;RandomAgent",
     help="A semicolon (;) separated list of agent types to use for the competition. You"
     " can also pass the special value default for the default builtin"
     " agents",
@@ -1811,7 +1860,7 @@ def run2021(
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -2087,7 +2136,7 @@ def tournament2020(
 )
 @click.option(
     "--log-negs/--no-neg-logs",
-    default=False,
+    default=True,
     help="Log all negotiations. Only effective if --debug is given",
 )
 @click.option(
@@ -2109,7 +2158,7 @@ def tournament2020(
 )
 @click.option(
     "--cw",
-    default=3,
+    default=2,
     type=int,
     help="Number of competitors to run at every world simulation. It must "
     "either be left at default or be a number > 1 and < the number "
@@ -2252,9 +2301,9 @@ def tournament2021(
 
     if non_competitors is None:
         non_competitors = (
-            DEFAULT_ONESHOT_FULL
+            DEFAULT_ONESHOT_NONCOMPETITORS
             if ttype == "oneshot"
-            else scml.scml2020.utils.DefaultAgents2021
+            else DEFAULT_2021_NONCOMPETITORS
         )
         non_competitor_params = tuple({} for _ in range(len(non_competitors)))
     print(f"Tournament will be run between {len(all_competitors)} agents: ")
@@ -2299,6 +2348,7 @@ def tournament2021(
     end_time = humanize_time(perf_counter() - start)
     display_results(results, "median", output)
     print(f"Finished in {end_time}")
+    save_run_info(name, results.path, "tournament")
 
 
 @main.command(help="Prints SCML version and NegMAS version")
