@@ -12,9 +12,11 @@ import numpy as np
 from negmas.situated import Adapter
 from negmas.helpers import instantiate, get_full_type_name
 
+
 from ..oneshot.ufun import OneShotUFun
 from ..oneshot.agent import OneShotAgent
 from ..oneshot.common import OneShotState, OneShotProfile
+from ..oneshot.mixins import OneShotUFunCreatorMixin
 
 from .components.trading import MarketAwareTradePredictionStrategy
 from .components.production import DemandDrivenProductionStrategy
@@ -33,6 +35,9 @@ class SCML2020Agent(Agent):
     def init(self):
         pass
 
+    def before_step(self):
+        pass
+
     def step(self):
         pass
 
@@ -220,6 +225,8 @@ class _SystemAgent(SCML2020Agent):
     def step(self):
         pass
 
+    def before_step(self):
+        pass
     def init(self):
         pass
 
@@ -337,8 +344,9 @@ class AWIHelper:
             exogenous_input_price=p,
             exogenous_output_quantity=qo,
             exogenous_output_price=po,
-            storage_cost=self._owner.get_storage_cost(),
-            delivery_penalty=self._owner.get_delivery_penalty(),
+            disposal_cost=self._owner.get_disposal_cost(),
+            shortfall_penalty=self._owner.get_shortfall_penalty(),
+            current_balance=self._owner.get_current_balance(),
         )
 
     @property
@@ -374,14 +382,14 @@ class AWIHelper:
         return self._owner.get_exogenous_output()[1]
 
     @property
-    def current_storage_cost(self) -> float:
+    def current_disposal_cost(self) -> float:
         """Cost of storing one unit (penalizes buying too much/ selling too little)"""
-        return self._owner.get_storage_cost()
+        return self._owner.get_disposal_cost()
 
     @property
-    def current_delivery_penalty(self) -> float:
+    def current_shortfall_penalty(self) -> float:
         """Cost of failure to deliver one unit (penalizes buying too little / selling too much)"""
-        return self._owner.get_delivery_penalty()
+        return self._owner.get_shortfall_penalty()
 
     @property
     def current_input_issues(self) -> List[Issue]:
@@ -450,6 +458,7 @@ class OneShotAdapter(
     MarketAwareTradePredictionStrategy,
     SCML2020Agent,
     Adapter,
+    OneShotUFunCreatorMixin
 ):
     """
     An adapter allowing agents developed for SCML-OneShot to run in
@@ -484,7 +493,7 @@ class OneShotAdapter(
     def init(self):
         self._oneshot_awi = AWIHelper(self)
         self._obj._awi = self._oneshot_awi
-        self._obj.ufun = self.make_ufun()
+        self._obj.ufun = self.make_ufun(add_exogenous=True)
         super().init()
         self._obj.init()
 
@@ -522,7 +531,7 @@ class OneShotAdapter(
         quantity = (1, self.awi.n_lines)
         return unit_price, time, quantity
 
-    def step(self):
+    def before_step(self):
         if self.awi.my_input_product == 0:
             pass
         else:
@@ -547,54 +556,18 @@ class OneShotAdapter(
                 unit_price=u,
                 controller=self._obj,
             )
-        self.utility_function = self.make_ufun()
-        super().step()
+        self.utility_function = self.make_ufun(add_exogenous=True)
+        super().before_step()
+        if hasattr(self._obj, "before_step"):
+            self._obj.before_step()
 
-    def make_ufun(self, add_exogenous=False):
-        iq = (
-            self._obj.awi.current_input_issues[QUANTITY]
-            if self._obj.awi.current_input_issues
-            else None
-        )
-        ip = (
-            self._obj.awi.current_input_issues[UNIT_PRICE]
-            if self._obj.awi.current_input_issues
-            else None
-        )
-        oq = (
-            self._obj.awi.current_output_issues[QUANTITY]
-            if self._obj.awi.current_output_issues
-            else None
-        )
-        op = (
-            self._obj.awi.current_output_issues[UNIT_PRICE]
-            if self._obj.awi.current_output_issues
-            else None
-        )
-        self.ufun = OneShotUFun(
-            ex_qin=self._obj.awi.current_exogenous_input_quantity if add_exogenous else 0,
-            ex_pin=self._obj.awi.current_exogenous_input_price if add_exogenous else 0,
-            ex_qout=self._obj.awi.current_exogenous_output_quantity if add_exogenous else 0,
-            ex_pout=self._obj.awi.current_exogenous_output_price if add_exogenous else 0,
-            production_cost=self._obj.awi.profile.cost,
-            storage_cost=self._obj.awi.current_storage_cost,
-            delivery_penalty=self._obj.awi.current_delivery_penalty,
-            input_penalty_scale=self._obj.awi.penalty_multiplier(True, None),
-            output_penalty_scale=self._obj.awi.penalty_multiplier(True, None),
-            input_agent=self._obj.awi.my_input_product == 0,
-            output_agent=self._obj.awi.my_output_product == self._obj.awi.n_products - 1,
-            input_product=self._obj.awi.my_input_product,
-            n_input_negs=self._obj.awi.n_input_negotiations,
-            n_output_negs=self._obj.awi.n_output_negotiations,
-            current_step=self._obj.awi.current_step,
-            input_qrange=(iq.min_value, iq.max_value) if iq else (0, 0),
-            input_prange=(ip.min_value, ip.max_value) if ip else (0, 0),
-            output_qrange=(oq.min_value, oq.max_value) if oq else (0, 0),
-            output_prange=(op.min_value, op.max_value) if op else (0, 0),
-            force_exogenous=self._obj.awi.is_exogenous_forced,
-            n_lines=self._obj.awi.n_lines,
-        )
-        return self.ufun
+    def step(self):
+        super().step()
+        if hasattr(self._obj, "before_step"):
+            self._obj.before_step()
+
+    def make_ufun(self, add_exogenous: bool):
+        return super().make_ufun(add_exogenous, in_adapter=True)
 
     def to_dict(self):
         return {
@@ -621,20 +594,20 @@ class OneShotAdapter(
         #     return None
         return self._obj.create_negotiator()
 
-    def get_storage_cost(self) -> float:
+    def get_disposal_cost(self) -> float:
         # penalty for buying too much
         return 0.0
 
-    def get_delivery_penalty_mean(self):
-        return self.get_delivery_penalty()
+    def get_shortfall_penalty_mean(self):
+        return self.get_shortfall_penalty()
 
-    def get_storage_cost_mean(self):
-        return self.get_storage_cost()
+    def get_disposal_cost_mean(self):
+        return self.get_disposal_cost()
 
-    def get_delivery_penalty_dev(self):
+    def get_shortfall_penalty_dev(self):
         return 0.0
 
-    def get_storage_cost_dev(self):
+    def get_disposal_cost_dev(self):
         return 0.0
 
     def get_profile(self):
@@ -642,15 +615,18 @@ class OneShotAdapter(
             cost=float(self.awi.profile.costs[:, self.awi.my_input_product].mean()),
             n_lines=self.awi.profile.n_lines,
             input_product=self.awi.my_input_product,
-            delivery_penalty_mean=self.get_delivery_penalty_mean(),
-            storage_cost_mean=self.get_storage_cost_mean(),
-            delivery_penalty_dev=self.get_delivery_penalty_dev(),
-            storage_cost_dev=self.get_storage_cost_dev(),
+            shortfall_penalty_mean=self.get_shortfall_penalty_mean(),
+            disposal_cost_mean=self.get_disposal_cost_mean(),
+            shortfall_penalty_dev=self.get_shortfall_penalty_dev(),
+            disposal_cost_dev=self.get_disposal_cost_dev(),
         )
 
-    def get_delivery_penalty(self):
+    def get_shortfall_penalty(self):
         return self.awi.trading_prices[self.awi.my_output_product]
 
+    def get_current_balance(self):
+        return self.awi.current_balance
+
     # todo: correct this
     def get_exogenous_output(self) -> Tuple[int, int]:
         return 0, 0
