@@ -1,8 +1,11 @@
 from negmas import LinearUtilityFunction
 
-from .indneg import IndependentNegotiationsAgent
+from .indneg import (
+    IndependentNegotiationsAgent,
+    MarketAwareIndependentNegotiationsAgent,
+)
 
-__all__ = ["BuyCheapSellExpensiveAgent"]
+__all__ = ["BuyCheapSellExpensiveAgent", "MarketAwareBuyCheapSellExpensiveAgent"]
 
 
 class BuyCheapSellExpensiveAgent(IndependentNegotiationsAgent):
@@ -12,3 +15,9 @@ class BuyCheapSellExpensiveAgent(IndependentNegotiationsAgent):
         if is_seller:
             return LinearUtilityFunction((1, 1, 10))
         return LinearUtilityFunction((1, -1, -10))
+
+
+class MarketAwareBuyCheapSellExpensiveAgent(
+    MarketAwareIndependentNegotiationsAgent, BuyCheapSellExpensiveAgent
+):
+    """An agent that tries to buy cheap and sell expensive but does not care about production scheduling."""
