@@ -27,7 +27,130 @@ __all__ = [
 
 
 class AWI(AgentWorldInterface):
-    """The Agent SCML2020World Interface for SCML2020 world allowing a single process per agent"""
+    """
+    The Agent SCML2020World Interface for SCML2020 world.
+
+    This class contains all the methods needed to access the simulation to
+    extract information which are divided into 5 groups:
+
+    Static World Information:
+        Information about the world and the agent that does not change over
+        time. These include:
+
+        A. Market Information:
+          - *n_products*: Number of products in the production chain.
+          - *n_processes*: Number of processes in the production chain.
+          - *n_competitors*: Number of other factories on the same production level.
+          - *all_suppliers*: A list of all suppliers by product.
+          - *all_consumers*: A list of all consumers by product.
+          - *catalog_prices*: A list of the catalog prices (by product).
+          - *inputs*: Inputs to every manufacturing process.
+          - *outputs*: Outputs to every manufacturing process.
+          - *is_system*: Is the given system ID corresponding to a system agent?
+          - *current_step*: Current simulation step (inherited from `negmas.situated.AgentWorldInterface` ).
+          - *n_steps*: Number of simulation steps (inherited from `negmas.situated.AgentWorldInterface` ).
+          - *relative_time*: fraction of the simulation completed (inherited from `negmas.situated.AgentWorldInterface`).
+          - *settings*: The system settings (inherited from `negmas.situated.AgentWorldInterface` ).
+
+        B. Agent Information:
+          - *profile*: Gives the agent profile including its production cost, number
+            of production lines, input product index, mean of its delivery
+            penalties, mean of its disposal costs, standard deviation of its
+            shortfall penalties and standard deviation of its disposal costs.
+            See `OneShotProfile` for full description. This information is private
+            information and no other agent knows it.
+          - *n_lines*: the number of production lines in the factory (private information).
+          - *is_first_level*: Is the agent in the first production level (i.e. it is an
+            input agent that buys the raw material).
+          - *is_last_level*: Is the agent in the last production level (i.e. it is an
+            output agent that sells the final product).
+          - *is_middle_level*: Is the agent neither a first level nor a last level agent
+          - *my_input_product*: The input product to the factory controlled by the agent.
+          - *my_output_product*: The output product from the factory controlled by the agent.
+          - *my_input_products*: All input products of a factory controlled by the agent.
+            Currently, it is always a list of one item. For future compatibility.
+          - *my_output_products*: All output products of a factory controlled by the agent.
+            Currently, it is always a list of one item. For future compatibility.
+          - *available_for_production*: Returns the line-step slots available for production.
+          - *level*: The production level which is numerically the same as the input product.
+          - *my_suppliers*: A list of IDs for all suppliers to the agent (i.e. agents
+            that can sell the input product of the agent).
+          - *my_consumers*: A list of IDs for all consumers to the agent (i.e. agents
+            that can buy the output product of the agent).
+          - *penalties_scale*: The scale at which to calculate disposal cost/delivery
+            penalties. "trading" and "catalog" mean trading and
+            catalog prices. "unit" means the contract's unit price
+            while "none" means that disposal cost/shortfall penalty
+            are absolute.
+          - *n_input_negotiations*: Number of negotiations with suppliers.
+          - *n_output_negotiations*: Number of negotiations with consumers.
+          - *state*: The full state of the agent ( `FactoryState` ).
+          - *current_balance*: The current balance of the agent
+          - *current_inventory*: The current inventory of the agent (quantity per product)
+
+    Dynamic World Information:
+        Information about the world and the agent that changes over time.
+
+        A. Market Information:
+          - *trading_prices*: The trading prices of all products. This information
+            is only available if `publish_trading_prices` is
+            set in the world.
+          - *exogenous_contract_summary*: A list of n_products tuples each giving
+            the total quantity and average price of
+            exogenous contracts for a product. This
+            information is only available if
+            `publish_exogenous_summary` is set in
+            the world.
+
+        B. Other Agents' Information:
+          - *reports_of_agent*: Gives all past financial reports of a given agent.
+            See `FinancialReport` for details.
+          - *reports_at_step*: Gives all reports of all agents at a given step.
+            See `FinancialReport` for details.
+
+        C. Current Negotiations Information:
+          - *current_input_issues*: The current issues for all negotiations to buy
+            the input product of the agent. If the agent
+            is at level zero, this will be empty.
+          - *current_output_issues*: The current issues for all negotiations to buy
+            the output product of the agent. If the agent
+            is at level n_products - 1, this will be empty.
+
+        D. Agent Information:
+          - *current_exogenous_input_quantity*: The total quantity the agent have
+            in its input exogenous contract.
+          - *current_exogenous_input_price*: The total price of the agent's
+            input exogenous contract.
+          - *current_exogenous_output_quantity*: The total quantity the agent have
+            in its output exogenous contract.
+          - *current_exogenous_output_price*: The total price of the agent's
+            output exogenous contract.
+          - *current_disposal_cost*: The disposal cost per unit item in the current
+            step.
+          - *current_shortfall_penalty*: The shortfall penalty per unit item in the current
+            step.
+
+    Actions:
+        A. Negotiation Control:
+          - *request_negotiations*: Requests a set of negotiations controlled by a
+            single controller.
+          - *request_negotiation*: Requests a negotiation controlled by a single
+            negotiator.
+
+        B. Production Control:
+          - *schedule_production*: Schedules production using one of the predefined
+            scheduling strategies.
+          - *order_production*: Orders production directly for the current step.
+          - *set_commands*: Sets production commands directly on the factory.
+          - *cancel_production*: Cancels a scheduled production command.
+
+    Services (All inherited from `negmas.situated.AgentWorldInterface`):
+      - *logdebug/loginfo/logwarning/logerror*: Logs to the world log at the given log level.
+      - *logdebug_agent/loginf_agnet/...*: Logs to the agent specific log at the given log level.
+      - *bb_query*: Queries the bulletin-board.
+      - *bb_read*: Read a section of the bulletin-board.
+
+    """
 
     # --------
     # Actions
@@ -134,7 +257,8 @@ class AWI(AgentWorldInterface):
         for p, neg, r in zip(partners, negotiators, results):
             if not r:
                 continue
-            controller.add_negotiator(neg)
+            if controller is not None:
+                controller.add_negotiator(neg)
             self._world._registered_negs[tuple(sorted([p, self.agent.id]))] += 1
         return any(results)
 
@@ -393,6 +517,16 @@ class AWI(AgentWorldInterface):
         """Receives the factory state"""
         return self._world.a2f[self.agent.id].state
 
+    @property
+    def current_balance(self):
+        """Current balance of the agent"""
+        return self.state.balance
+
+    @property
+    def current_inventory(self):
+        """Current inventory of the agent"""
+        return self.state.inventory
+
     def reports_of_agent(self, aid: str) -> Dict[int, FinancialReport]:
         """Returns a dictionary mapping time-steps to financial reports of the given agent"""
         return self.bb_read("reports_agent", aid)
@@ -457,7 +591,7 @@ class AWI(AgentWorldInterface):
     @property
     def n_competitors(self) -> int:
         """Returns the number of factories/agents in the same production level"""
-        return len(self._world.consumers[self.my_output_product])
+        return len(self._world.consumers[self.my_output_product]) - 1
 
     @property
     def my_input_product(self) -> int:
@@ -524,6 +658,33 @@ class AWI(AgentWorldInterface):
         """Number of processes in the world"""
         return self.state.n_processes
 
+    @property
+    def is_first_level(self):
+        """
+        Whether this agent is in the first production level
+        """
+        return self.my_input_product == 0
+
+    @property
+    def is_last_level(self):
+        """
+        Whether this agent is in the last production level
+        """
+        return self.my_output_product == self.n_products - 1
+
+    @property
+    def level(self):
+        """The production level which is the index of the process for
+        this factory (or the index of its input product)"""
+        return self.my_input_product
+
+    @property
+    def is_middle_level(self):
+        """
+        Whether this agent is in neither in the first nor in the last level
+        """
+        return 0 < self.my_input_product < self.n_products - 2
+
     def is_system(self, aid: str) -> bool:
         """
         Checks whether an agent is a system agent or not
