@@ -258,13 +258,15 @@ class Factory:
         possible = min(repeats, len(steps))
         if possible < repeats:
             return np.empty(shape=0, dtype=int), np.empty(shape=0, dtype=int)
-        if method.startswith("l"):
-            steps, lines = steps[-possible + 1 :], lines[-possible + 1 :]
-        elif method == "all":
-            pass
-        else:
-            steps, lines = steps[:possible], lines[:possible]
-
+        try:
+            if method.startswith("l"):
+                steps, lines = steps[-possible + 1 :], lines[-possible + 1 :]
+            elif method == "all":
+                pass
+            else:
+                steps, lines = steps[:possible], lines[:possible]
+        except:
+            return np.empty(shape=0, dtype=int), np.empty(shape=0, dtype=int)
         return steps, lines
 
     def cancel_production(self, step: int, line: int) -> bool:
@@ -407,7 +409,7 @@ class Factory:
         if available + quantity >= 0:
             self._inventory[product] += quantity
             self.inventory_changes[product] += quantity
-            return quantity if quantity > 0 else -quantity
+            return int(quantity if quantity > 0 else -quantity)
         # we have an inventory breach here. We know that quantity < 0
         assert quantity < 0
         quantity = -quantity
@@ -420,7 +422,7 @@ class Factory:
             self.pay(to_pay, no_bankruptcy, no_borrowing)
             self._inventory[product] = 0
             self.inventory_changes[product] -= available
-            return available
+            return int(available)
         # we have an inventory breach and should try to buy missing quantity from the spot market
         effective_unit = self.spot_price(product, spot_price)
         effective_total = (quantity - available) * effective_unit
@@ -435,7 +437,7 @@ class Factory:
         self._inventory[product] += paid_for
         self.inventory_changes[product] += paid_for
         self.store(product, -quantity, False, 0.0, True, True)
-        return paid_for
+        return int(paid_for)
 
     def buy(
         self,
