@@ -3,8 +3,8 @@ import random
 import hypothesis.strategies as st
 from hypothesis import example
 from hypothesis import given
-from hypothesis import settings
 from hypothesis import reproduce_failure
+from hypothesis import settings
 from negmas import save_stats
 from negmas.helpers import unique_name
 from pytest import mark
@@ -14,10 +14,13 @@ from scml.oneshot.agents import RandomOneShotAgent
 from scml.oneshot.agents import SyncRandomOneShotAgent
 from scml.scml2020 import BuyCheapSellExpensiveAgent
 from scml.scml2020 import DoNothingAgent
+from scml.scml2020 import SCML2020Agent
 from scml.scml2020 import RandomAgent
 from scml.scml2020 import SCML2021World
+from scml.scml2020 import SatisficerAgent
 from scml.scml2020 import is_system_agent
 from scml.scml2020.agents.decentralizing import DecentralizingAgent
+from scml.scml2020.utils import anac2021_collusion
 
 random.seed(0)
 
@@ -35,7 +38,7 @@ def generate_world(
     n_steps=10,
     n_agents_per_process=2,
     n_lines=10,
-    initial_balance=10_000,
+    initial_balance=None,
     buy_missing_products=True,
     **kwargs,
 ):
@@ -136,6 +139,7 @@ def test_can_run_with_multiple_agent_types(agent_types, buy_missing, n_processes
     n_processes=st.integers(2, 4),
     initial_balance=st.sampled_from([50, 10_000, 10_000_000]),
 )
+@settings(deadline=500_000)
 def test_nothing_happens_with_do_nothing(buy_missing, n_processes, initial_balance):
     world = generate_world(
         [DoNothingAgent],
@@ -244,14 +248,17 @@ def test_agents_go_bankrupt(n_processes):
         #     )
 
 
-def test_generate():
+@mark.parametrize("n_processes", [2, 3, 4, 5, 6])
+def test_generate(n_processes):
     world = SCML2021World(
         **SCML2021World.generate(
-            agent_types=DoNothingAgent, n_steps=10, n_processes=4, initial_balance=None
+            agent_types=DoNothingAgent,
+            n_steps=50,
+            n_processes=n_processes,
+            initial_balance=None,
         )
     )
     world.run()
-    assert True
 
 
 def test_a_tiny_world():
@@ -303,6 +310,12 @@ def test_graphs_lead_to_no_unknown_nodes():
         st.sampled_from(oneshot_types + types), unique=True, min_size=2, max_size=6
     )
 )
+@example(
+    atype=[
+        scml.oneshot.agents.greedy.GreedySyncAgent,
+        scml.scml2020.agents.random.RandomAgent,
+    ],
+)
 @settings(deadline=300_000, max_examples=30)
 def test_adapter(atype):
     world = SCML2021World(
@@ -313,17 +326,167 @@ def test_adapter(atype):
     )
     world.run()
 
-def test_adapter_example():
-    world = SCML2021World(
-        **SCML2021World.generate(
-            agent_types=[
-                "scml.oneshot.agents.SingleAgreementAspirationAgent", 
-                "scml.scml2020.DecentralizingAgent"
-                ], 
-            n_steps=10
-        ),
-        construct_graphs=False,
-        no_logs=True,
-        compact=True,
+
+def test_production_cost_increase():
+    from scml.scml2020 import SCML2021World
+    from scml.scml2020.agents import DecentralizingAgent
+
+    NPROCESSES = 5
+    costs = [[] for _ in range(NPROCESSES)]
+    for _ in range(100):
+        world = SCML2021World(
+            **SCML2021World.generate(
+                DecentralizingAgent,
+                n_agents_per_process=10,
+                n_processes=NPROCESSES,
+            ),
+            compact=True,
+            no_logs=True,
+        )
+        for aid in world.agent_profiles.keys():
+            if is_system_agent(aid):
+                continue
+            profile = world.agent_profiles[aid]
+            costs[profile.input_products[0]].append(
+                profile.costs[:, profile.input_products[0]].mean()
+            )
+    mean_costs = [sum(_) / len(_) for _ in costs]
+    assert all(
+        [
+            b > (0.5 * (i + 2) / (i + 1)) * a
+            for i, (a, b) in enumerate(zip(mean_costs[:-1], mean_costs[1:]))
+        ]
+    ), f"non-ascending costs {mean_costs}"
+
+
+@mark.parametrize("n_processes", [2, 3, 4])
+def test_satisficer(n_processes):
+    world = generate_world(
+        [SatisficerAgent],
+        buy_missing_products=True,
+        n_processes=n_processes,
+        compact=COMPACT,
+        no_logs=NOLOGS,
     )
     world.run()
+    save_stats(world, world.log_folder)
+
+
+N_AGENTS_PER_COMPETITORS = 3
+
+
+class MyColluders(DoNothingAgent):
+    # define a class-level variable to share the information
+    my_friends = dict()
+
+    def init(self):
+        # share any information that will be static for the whole competition
+        self.my_friends[self.id] = dict(
+            level=self.awi.my_input_product, initial_balance=self.awi.current_balance
+        )
+
+    def before_step(self):
+        # here you can access my_frieds and it will be fully populated.
+        # any thing shared in `init()` or `step()` is available here
+        assert self.id in self.my_friends
+        if self.awi.current_step > 0:
+            for v in self.my_friends.values():
+                assert "last_step_balance" in v
+                if v["level"] < self.awi.my_current_input:
+                    assert "current_step_balance" in v
+
+        # information shared here is consistently available to agents at
+        # higher levels
+        self.my_friends[self.id].update(dict(current_balance=self.awi.current_balance))
+
+    def step(self):
+
+        assert self.id in self.my_friends
+        for v in self.my_friends.values():
+            assert "current_step_balance" in v
+
+        # share any information that is available by the end of the step
+        self.my_friends[self.id].update(
+            dict(
+                last_step_balance=self.awi.current_balance,
+                last_step_inventory=self.awi.current_inventory,
+            )
+        )
+
+
+def test_colluding_agents_find_each_other():
+    anac2021_collusion(
+        competitors=[MyColluders, DoNothingAgent],
+        n_agents_per_competitor=N_AGENTS_PER_COMPETITORS,
+        n_configs=1,
+        n_processes=2,
+        n_steps=4,
+    )
+
+
+@mark.parametrize(
+    ["method", "n_agents", "n_processes", "n_steps"],
+    [
+        ("guaranteed_profit", 1, 2, 5),
+        ("profitable", 1, 2, 5),
+        ("guaranteed_profit", 1, 2, 8),
+        ("profitable", 1, 2, 8),
+        ("guaranteed_profit", 2, 2, 8),
+        ("profitable", 2, 2, 8),
+        ("guaranteed_profit", 5, 2, 8),
+        ("profitable", 5, 2, 8),
+        ("guaranteed_profit", 1, 2, 16),
+        ("profitable", 1, 2, 16),
+        ("guaranteed_profit", 2, 2, 16),
+        ("profitable", 2, 2, 16),
+        ("guaranteed_profit", 5, 2, 16),
+        ("profitable", 5, 2, 16),
+        ("guaranteed_profit", 1, 3, 8),
+        ("profitable", 1, 3, 8),
+        ("guaranteed_profit", 2, 3, 8),
+        ("profitable", 2, 3, 8),
+        ("guaranteed_profit", 5, 3, 8),
+        ("profitable", 5, 3, 8),
+        ("guaranteed_profit", 1, 3, 16),
+        ("profitable", 1, 3, 16),
+        ("guaranteed_profit", 2, 3, 16),
+        ("profitable", 2, 3, 16),
+        ("guaranteed_profit", 5, 3, 16),
+        ("profitable", 5, 3, 16),
+        # ("guaranteed_profit", 2, 3, 80),
+        # ("profitable", 2, 3, 80),
+    ],
+)
+def test_satisficer_n_agent_per_level(method, n_agents, n_processes, n_steps):
+    from scml.scml2020 import SCML2021World
+    from pathlib import Path
+    from negmas.situated import save_stats
+    from negmas.helpers import force_single_thread
+
+    force_single_thread(True)
+    world = SCML2021World(
+        **SCML2021World.generate(
+            [SatisficerAgent] * n_processes,
+            n_agents_per_process=n_agents,
+            n_processes=n_processes,
+            n_steps=n_steps,
+            method=method,
+            random_agent_types=False,
+            neg_step_time_limit=float("inf"),
+            neg_time_limit=float("inf"),
+        ),
+        compact=True,
+        no_logs=True,
+        avoid_ultimatum=True,
+        end_negotiation_on_refusal_to_propose=True,
+        name=f"Satisficer1{method}-a{n_agents}p{n_processes}s{n_steps}",
+    )
+    world.run()
+    world.save_negotiations = True
+    save_stats(
+        world,
+        log_dir=Path.home() / "negmas" / "logs" / "scml" / "scml2021" / world.name,
+    )
+    force_single_thread(False)
+
+    assert True
